optim2by2pooled <- function(p,p1,q1,p0,ptd,fdr) {

#
# This function computes the first and second stage critical values and sample sizes of the two-stage 
# design for allele-based test (2X2 tables) with minimal genotyping burden for given PTD 
# and pFDR, when data are POOLED. The calculations are based on the COST-EFFECTIVE APPROXIMATION.
#
# This function uses an algorithm given in the article
#  "J\'{o}zsef Buksz\'{a}r and Edwin J. C. G. van den Oord
#   Optimization of two-stage genetic designs where data are pooled 
#   usingan accurate and efficient approximation for Pearson's statistic,
#   Biometrics, 2005 (submitted)"
#
# Questions about the codes should be addressed to
#  Jo'zsef Buksza'r 
#  email: jbukszar@vcu.edu
#
#
# 
# Input parameters:
#       p  =  proportion of controls in the sample size
#       p1 =  probability that a control falls in category 1 under the alternative hypothesis
#       q1 =  probability that a case falls in category 1 under the alternative hypothesis
#       p0 =  proportion of true null hypotheses
#      ptd =  proportion of true discoveries (PTD) (this can also be interpreted as average power)
#      fdr =  positive false discovery rate (pFDR).
#
#  PACKAGE NEEDED: mvtnorm
#  FUNCTIONS NEEDED: myoptimalnonnew, refineF, critcal (directly)
#                    n2minsimpF, condevalnonsimp, rminimizernon, Gritsimp, prejnon, prejectsimp (indirectly)
#
#  IN OUTPUT, SAMPLE SIZE IS THE NUMBER OF INDIVIDUALS RECRUITED FOR THE EXPERIMENT !!!                              
#                  

   res <- myoptimalnonnew(p,p1,q1,p0,ptd,fdr)

   c100 <- res[2]
   c200 <- res[3]
   n1nonp <- 0.5*res[4]
   n2nonp <- 0.5*res[5]


   n1 <- 2*round( (0.91254*n1nonp + 0.01254*n2nonp) )
   n2 <- 2*round( (-0.61254*n1nonp + 0.98746*n2nonp) )

  q <- 1 - p
  p2 <- 1-p1
  q2 <- 1-q1

  mubase <- (p1-q1)*sqrt( p*q/((p1*p+q1*q)*(p2*p+q2*q)) )

  # Calculating szigma

  aux1 <- ((p1-q1)*q)/(2*(p1*p+q1*q)) + 1
  aux2 <- ((p2-q2)*q)/(2*(p2*p+q2*q)) + 1
  bux1 <- -((p1-q1)*p)/(2*(p1*p+q1*q)) + 1
  bux2 <- -((p2-q2)*p)/(2*(p2*p+q2*q)) + 1

  B11 <- (aux1*aux1*p*q1*q2 + bux1*bux1*q*p1*p2) / (p1*p+q1*q)
  B22 <- (aux2*aux2*p*q1*q2 + bux2*bux2*q*p1*p2) / (p2*p+q2*q)
  B12 <- -(aux1*aux2*p*q1*q2 + bux1*bux2*q*p1*p2) / sqrt( (p1*p+q1*q)*(p2*p+q2*q) )
  B21 <- B12

  bb <- -(B11+B22)
  cc <- B11*B22-B12*B21

  eig1 <- ( -bb + sqrt(bb*bb-4*cc) ) / 2
  eig2 <- ( -bb - sqrt(bb*bb-4*cc) ) / 2

  szigma <- sqrt( max(eig1,eig2) )

  # End of szigma calculation


   res <- critcal(c100,c200,n1,n2,mubase,szigma,p0,ptd,fdr)

   c10 <- res[1]
   c20 <- res[2]
   gb0 <- res[3]

#   print( " Initial genburd (half), c1, c2, n1 (half), n2 (half) = " )
#   print( c(0.5*gb0, c10, c20, 0.5*n1, 0.5*n2) )

   print( " Refinement starts: " )
   print( " ------------ " )

    refineF(c10,c20,n1,n2,p,p1,q1,p0,ptd,fdr)
}



myoptimalnonnew <- function(p,p1,q1,p0,ptd,fdr) {    

# IT RETURNS VERY ACCURATE RESULTS!
# It finds the minimal genotyping burden calculated by "condevalnonsimp"
# for the variables c (first stage critical value) and r (=sqrt(rn/(rn+rnprime)).
#
# PACKAGE NEEDED: -
# FUNCTION NEEDED: condevalnonsimp, rminimizernon, (directly)
#                  Gcritsimp (indirectly)

  q <- 1 - p
  p2 <- 1-p1
  q2 <- 1-q1

  mubase <- (p1-q1)*sqrt( p*q/((p1*p+q1*q)*(p2*p+q2*q)) )

  # Calculating szigma
  aux1 <- ((p1-q1)*q)/(2*(p1*p+q1*q)) + 1
  aux2 <- ((p2-q2)*q)/(2*(p2*p+q2*q)) + 1
  bux1 <- -((p1-q1)*p)/(2*(p1*p+q1*q)) + 1
  bux2 <- -((p2-q2)*p)/(2*(p2*p+q2*q)) + 1

  B11 <- (aux1*aux1*p*q1*q2 + bux1*bux1*q*p1*p2) / (p1*p+q1*q)
  B22 <- (aux2*aux2*p*q1*q2 + bux2*bux2*q*p1*p2) / (p2*p+q2*q)
  B12 <- -(aux1*aux2*p*q1*q2 + bux1*bux2*q*p1*p2) / sqrt( (p1*p+q1*q)*(p2*p+q2*q) )
  B21 <- B12

  bb <- -(B11+B22)
  cc <- B11*B22-B12*B21

  eig1 <- ( -bb + sqrt(bb*bb-4*cc) ) / 2
  eig2 <- ( -bb - sqrt(bb*bb-4*cc) ) / 2

  szigma <- sqrt( max(eig1,eig2) )

  szigmaold <- sqrt( (p1*p2*q+q1*q2*p) / ((p1*p+q1*q)*(p2*p+q2*q)) )

  # End of szigma calculation

  aux <- (ptd*fdr) / (p0*(1-fdr))
  b <- aux * (1-p0)
  aux = qnorm(0.5*b)
  cmax <- aux * aux

# --------------------------

  cold <- 4
  r0 <- 0.2
  cdelta <- 0.00001

  old <- 0
  new <- 0
  cnew <- 0

  cveryold <- 0
  veryold <- 0


  res <- rminimizernon(cold,r0,p0,ptd,fdr,mubase,szigma)   
  old = res[1]
  r0 = res[2]
#  print(res)
  res <- rminimizernon(cold+cdelta,r0,p0,ptd,fdr,mubase,szigma) 
  new = res[1]
#  print(res)

  if (new < old) cstep <- 0.5 else cstep <- -0.5
  cnew = cold + cstep
  res <- rminimizernon(cnew,r0,p0,ptd,fdr,mubase,szigma) 
  new = res[1]
#  print(res)

  cveryold = cold
  veryold = old
  while (new < old) {
      cveryold = cold
      veryold = old
      cold = cnew
       old = new
      cnew = cold + cstep
      res <- rminimizernon(cnew,r0,p0,ptd,fdr,mubase,szigma) 
      new = res[1]
      r0 = res[2]
#      print(res)
  }

  if (cveryold < cnew) {
     a <- cveryold
     b <- cnew
     fva <- veryold
     fvb <- new
  } else {
     b <- cveryold
     a <- cnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  res <- rminimizernon(c,r0,p0,ptd,fdr,mubase,szigma) 
  fvc <- res[1]

  iter <- 0
  while ( b-a > 0.00001 ) {
     iter = iter + 1

     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        res <- rminimizernon(c,r0,p0,ptd,fdr,mubase,szigma)
        fvc = res[1]
        r0 = res[2]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            res <- rminimizernon(c,r0,p0,ptd,fdr,mubase,szigma)
            fvc = res[1]
            r0 = res[2]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            res <- rminimizernon(a,r0,p0,ptd,fdr,mubase,szigma)
            fva = res[1]
            res <- rminimizernon(b,r0,p0,ptd,fdr,mubase,szigma)
            fvb = res[1]
            r0 = res[2]
         }
     }    

  } # end while

  if ( fvc > fva ) {
     crit1 <- a
     gb <- fva
  } else {
      if ( fvc > fvb ) {
          crit1 <- b
          gb <- fvb
      } else {
          crit1 <- c
          gb <- fvc
      }
  }    

  resul <- rminimizernon(crit1,r0,p0,ptd,fdr,mubase,szigma)
  rold <- resul[2]
  res <- condevalnonsimp(crit1,rold,p0,ptd,fdr,mubase,szigma)

  res
}



refineF <- function(c10,c20,n10,n20,p,p1,q1,p0,ptd,fdr) {

#  FOR ALLELE BASED TESTS (HALF SAMPLE SIZE FOR 2 X 2 TABLES)
#
#  PACKAGE NEEDED: mvtnorm
#  FUNCTION NEEDED: n2minsimpF, critcal 
#                   Gritsimp, prejnon, prejectsimp (indirectly)
#                  

  q <- 1 - p
  p2 <- 1-p1
  q2 <- 1-q1

  mubase <- (p1-q1)*sqrt( p*q/((p1*p+q1*q)*(p2*p+q2*q)) )

  # Calculating szigma
  aux1 <- ((p1-q1)*q)/(2*(p1*p+q1*q)) + 1
  aux2 <- ((p2-q2)*q)/(2*(p2*p+q2*q)) + 1
  bux1 <- -((p1-q1)*p)/(2*(p1*p+q1*q)) + 1
  bux2 <- -((p2-q2)*p)/(2*(p2*p+q2*q)) + 1

  B11 <- (aux1*aux1*p*q1*q2 + bux1*bux1*q*p1*p2) / (p1*p+q1*q)
  B22 <- (aux2*aux2*p*q1*q2 + bux2*bux2*q*p1*p2) / (p2*p+q2*q)
  B12 <- -(aux1*aux2*p*q1*q2 + bux1*bux2*q*p1*p2) / sqrt( (p1*p+q1*q)*(p2*p+q2*q) )
  B21 <- B12

  bb <- -(B11+B22)
  cc <- B11*B22-B12*B21

  eig1 <- ( -bb + sqrt(bb*bb-4*cc) ) / 2
  eig2 <- ( -bb - sqrt(bb*bb-4*cc) ) / 2

  szigma <- sqrt( max(eig1,eig2) )

  szigmaold <- sqrt( (p1*p2*q+q1*q2*p) / ((p1*p+q1*q)*(p2*p+q2*q)) )

  # End of szigma calculation


  old <- 0
  new <- 0
  rnew <- 0

  rold <- n10
  rveryold <- 0
  veryold <- 0


  res <- n2minsimpF(c10,c20,rold,n20,mubase,szigma,p0,ptd,fdr)
  old = res[1]

  res <- n2minsimpF(c10,c20,rold+1,res[2],mubase,szigma,p0,ptd,fdr)
  new = res[1]

  if (new < old) rstep <- 4 else rstep <- -4
  rnew = rold + rstep
  res <- n2minsimpF(c10,c20,rnew,res[2],mubase,szigma,p0,ptd,fdr)
  new = res[1]


  rveryold = rold
  veryold = old
  while (new < old) {
      rveryold = rold
      veryold = old
      rold = rnew
       old = new
      rnew = rold + rstep
      res <- n2minsimpF(c10,c20,rnew,res[2],mubase,szigma,p0,ptd,fdr)
      new = res[1]

  }

  if (rveryold < rnew) {
     a <- rveryold
     b <- rnew
     fva <- veryold
     fvb <- new
  } else {
     b <- rveryold
     a <- rnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  res <- n2minsimpF(c10,c20,c,res[2],mubase,szigma,p0,ptd,fdr)
  fvc <- res[1]

  iter <- 0
  while ( b-a > 4 ) {
     iter = iter + 1
     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        res <- n2minsimpF(c10,c20,c,res[2],mubase,szigma,p0,ptd,fdr)
        fvc = res[1]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            res <- n2minsimpF(c10,c20,c,res[2],mubase,szigma,p0,ptd,fdr)
            fvc = res[1]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            res <- n2minsimpF(c10,c20,a,res[2],mubase,szigma,p0,ptd,fdr)
            fva = res[1]
            res <- n2minsimpF(c10,c20,b,res[2],mubase,szigma,p0,ptd,fdr)
            fvb = res[1]
         }
     } 

  } # end while

  if ( fvc > fva ) {
     value <- c(fva,a)
  } else {
      if ( fvc > fvb ) {
          value <- c(fvb,b)
      } else {
          value <- c(fvc,c)
      }
  }    

  genburd <- value[1]
  n1 <- value[2]
  res <- n2minsimpF(c10,c20,n1,res[2],mubase,szigma,p0,ptd,fdr)
  n2 = res[2]
  res <- critcal(c10,c20,n1,n2,mubase,szigma,p0,ptd,fdr)
  c1 <- res[1]
  c2 <- res[2]

  print( " END RESULT: " )

  print(" First and second stage critical value = ")
  print( c(c1, c2) )

  print(" First and second stage sample size = ")
  print( c(n1/2, n2/2) )

  print(" genotyping burden = ")
  print( genburd/2 )

}



critcal <- function(c10,c20,n1,n2,mubase,szigma,p0,ptd,fdr) {

#  FOR 2 X 2 TABLES
#  BASED ON COSTEFFECTIVE METHOD
#  Calculates the first and second stage critical values for the 
#  first and second stage sample sizes so that both constraints hold. 
#
#  PACKAGE NEEDED: mvtnorm
#  FUNCTION NEEDED: prejnon, prejectsimp
#                  

    n0 <- n1 + n2
    r <- sqrt(n1/n0)

    a <- ptd
    aux <- (ptd*fdr) / (p0*(1-fdr))
    b <- aux * (1-p0)

    delta <- 0.00001
    c1 <- c10
    c2 <- c20

    auxpn <- prejnon(c1,c2,r)
    auxpr <- prejectsimp(c1,c2,n1,n2,mubase,szigma)
    dist <- c(b - auxpn,a - auxpr)

    A <- rep(0,4)
    dim(A) = c(2,2)

    while( 1000*abs(dist[1]) + abs(dist[2]) > 0.0000001 ) {
       A[1,] = c( (prejnon(c1+delta,c2,r) - auxpn)/delta, (prejnon(c1,c2+delta,r) - auxpn)/delta )
       A[2,] = c( (prejectsimp(c1+delta,c2,n1,n2,mubase,szigma) - auxpr)/delta, (prejectsimp(c1,c2+delta,n1,n2,mubase,szigma) - auxpr)/delta )

       step <- solve(A,dist)   
       c1 = c1 + step[1]
       c2 = c2 + step[2]
  
       auxpn = prejnon(c1,c2,r)
       auxpr = prejectsimp(c1,c2,n1,n2,mubase,szigma)
       dist = c(b - auxpn,a - auxpr)

    }

    genburd <- n1 + n2*( p0*(1-pchisq(c1,1)) +(1-p0)*Gcritsimp(c1,n1,mubase,szigma) ) 

    value <- c(c1,c2,genburd)

    value
}



n2minsimpF <- function(c10,c20,n1,n20,mubase,szigma,p0,ptd,fdr) {

#  FOR ALLELE BASED TESTS (HALF SAMPLE SIZE FOR 2 X 2 TABLES)
#  BASED ON COSTEFFECTIVE APPROXIMATION
#
#  PACKAGE NEEDED: mvtnorm
#  FUNCTION NEEDED: critcal
#            prejnon, prejectsimp (indirectly)
#                  

  old <- 0
  new <- 0
  rnew <- 0

  rold <- n20
  rveryold <- 0
  veryold <- 0

  res <- critcal(c10,c20,n1,rold,mubase,szigma,p0,ptd,fdr)
  old = res[3]

  res <- critcal(c10,c20,n1,rold+1,mubase,szigma,p0,ptd,fdr)
  new = res[3]

  if (new < old) rstep <- 8 else rstep <- -8
  rnew = rold + rstep
  res <- critcal(c10,c20,n1,rnew,mubase,szigma,p0,ptd,fdr)
  new = res[3]

#print(rstep)

  rveryold = rold
  veryold = old
  while (new < old) {
      rveryold = rold
      veryold = old
      rold = rnew
       old = new
      rnew = rold + rstep
      res <- critcal(c10,c20,n1,rnew,mubase,szigma,p0,ptd,fdr)
      new = res[3]

  }

  if (rveryold < rnew) {
     a <- rveryold
     b <- rnew
     fva <- veryold
     fvb <- new
  } else {
     b <- rveryold
     a <- rnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  resc <- critcal(c10,c20,n1,c,mubase,szigma,p0,ptd,fdr)
  fvc <- resc[3]

  iter <- 0
  while ( b-a > 4 ) {
     iter = iter + 1
     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        resc <- critcal(c10,c20,n1,c,mubase,szigma,p0,ptd,fdr)
        fvc = resc[3]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            resc <- critcal(c10,c20,n1,c,mubase,szigma,p0,ptd,fdr)
            fvc = resc[3]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            resa <- critcal(c10,c20,n1,a,mubase,szigma,p0,ptd,fdr)
            fva = resa[3]
            resb <- critcal(c10,c20,n1,b,mubase,szigma,p0,ptd,fdr)
            fvb = resb[3]
         }
     } 

  } # end while

  if ( fvc > fva ) {
     value <- c(fva,a)
  } else {
      if ( fvc > fvb ) {
          value <- c(fvb,b)
      } else {
          value <- c(fvc,c)
      }
  }    


  value

}



condevalnonsimp <- function(c,r,p0,ptd,fdr,mubase,szigma) {    

# ASSUMES NONPOOLED DATA
# For a first stage critical value and r=sqrt(rn/rntot), it calculates 
# the second stage critical value and sample size and the genotyping burden
# in such a way that the two conditions in the NLP hold. 
#
# PACKAGE NEEDED: -
# FUNCTION NEEDED:  Gcritsimp

    gamma <- mubase

    a <- ptd
    aux <- (ptd*fdr) / (p0*(1-fdr))
    b <- aux * (1-p0)
    rcoeff <- 1/(r*r) - 1

    aux <- qnorm(  b/( 4*pnorm(-sqrt(c)) )  )
    cs <- aux * aux

    mrootcs <- -sqrt(cs) / szigma
    mrootc <- -sqrt(c) / szigma
    gamsz <- gamma / szigma
    gamperr <- gamsz * sqrt(rcoeff)

    preject1 <- function(nx) { 
        rootn <- sqrt(nx)
        value <- ( pnorm(mrootcs+gamperr*rootn) + pnorm(mrootcs-gamperr*rootn) ) * ( pnorm(mrootc+gamsz*rootn) + pnorm(mrootc-gamsz*rootn) )  - a
        value
    }
    lo <- preject1(1)
    up <- preject1(100000)
    if (lo*up>0 || is.nan(lo*up)) {
        print(c(lo,up))
        print(c(c,r,cs))
        genburd <- 10000000
    } else {
      o   <- uniroot(preject1, lower = 1, upper = 100000, tol = 0.00001)
      rn <- o$root[1]  # The second condition holds with this rn (first stage sample size) and 
      rnprime <- rcoeff * rn           # this rnprime (second stage sample size).

      genburd <- rn + rnprime*( p0*(1-pchisq(c,1)) +(1-p0)*Gcritsimp(c,rn,mubase,szigma) ) 

    }
                                           
    value <- c(genburd,c,cs,rn,rnprime)

    value

}



rminimizernon <- function(cold,r0,p0,ptd,fdr,mubase,szigma) {    

# For a given first stage critical value, cold, it calculates r
# that minimizes the genotyping burden in the unconstrained NLP.
#
# PACKAGE NEEDED: -
# FUNCTION NEEDED: condevalnonsimp
#                  Gcritsimp (indirectly)


  rdelta <- 0.000001

  old <- 0
  new <- 0
  rnew <- 0

  rold <- r0
  rveryold <- 0
  veryold <- 0

  res <- condevalnonsimp(cold,rold,p0,ptd,fdr,mubase,szigma)
  old = res[1]
  res <- condevalnonsimp(cold,rold+rdelta,p0,ptd,fdr,mubase,szigma)
  new = res[1]

  if (new < old) rstep <- 0.1 else rstep <- -0.1
  rnew = rold + rstep
  res <- condevalnonsimp(cold,rnew,p0,ptd,fdr,mubase,szigma)
  new = res[1]

  rveryold = rold
  veryold = old
  while (new < old) {
      rveryold = rold
      veryold = old
      rold = rnew
       old = new
      rnew = rold + rstep
      res <- condevalnonsimp(cold,rnew,p0,ptd,fdr,mubase,szigma)
      new = res[1]
  }

  if (rveryold < rnew) {
     a <- rveryold
     b <- rnew
     fva <- veryold
     fvb <- new
  } else {
     b <- rveryold
     a <- rnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  res <- condevalnonsimp(cold,c,p0,ptd,fdr,mubase,szigma)
  fvc <- res[1]

  iter <- 0
  while ( b-a > 0.00001 ) {
     iter = iter + 1
     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        res <- condevalnonsimp(cold,c,p0,ptd,fdr,mubase,szigma)
        fvc = res[1]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            res <- condevalnonsimp(cold,c,p0,ptd,fdr,mubase,szigma)
            fvc = res[1]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            res <- condevalnonsimp(cold,a,p0,ptd,fdr,mubase,szigma)
            fva = res[1]
            res <- condevalnonsimp(cold,b,p0,ptd,fdr,mubase,szigma)
            fvb = res[1]
         }
     }    

  } # end while

  if ( fvc > fva ) {
     value <- c(fva,a)
  } else {
      if ( fvc > fvb ) {
          value <- c(fvb,b)
      } else {
          value <- c(fvc,c)
      }
  }    

  value

}



Gcritsimp <- function(c,rn,mubase,szigma) {    

# BASED ON THE COST-EFFECTIVE APPROXIMATION!
# Calculates the probability of rejecting a marker in a single stage when the critical value is c,
# the sample size is rn under H_1, i.e. when the probability table is given by p1 and q1.
# Number of controls is p*rn, number of cases is q*rn.
#
# FUNCTION NEEDED: -
     
      coeffo <- 1 / (szigma*szigma)

      nonco <- coeffo * mubase * mubase * rn
      value <- 1 - pchisq(coeffo*c, 1, ncp=nonco)

      value
}



prejnon <- function(csquare,ccsillsquare,r) {    

# Calculates the probability of rejecting a marker WITHOUT EFFECT on two stage
# pooled data.
#
#  csquare and ccsillsquare are the critical values in the first and second stage, respectively.
#  r <- sqrt(rn/(rn+rnprime))
#
# PACKAGE NEEDED: mvtnorm
# FUNCTION NEEDED: -

      c <- sqrt(csquare)
      ccsill <- sqrt(ccsillsquare)

      x1 <- c
      x2 <- -c
      y1 <- ccsill
      y2 <- -ccsill


# The calculation

      sigma<-diag(2)

      sigma[2,1]<-r
      sigma[1,2]<-r

      prob<-rep(0,4)
      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(x1,y1), upper=c(Inf,Inf))
      prob[1] <- resa[1]

      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(-Inf,y1), upper=c(x2,Inf))
      prob[2] <- resa[1]

      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(x1,-Inf), upper=c(Inf,y2))
      prob[3] <- resa[1]

      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(-Inf,-Inf), upper=c(x2,y2))
      prob[4] <- resa[1]

      value <- sum(prob)

      value

}



prejectsimp <- function(csquare,ccsillsquare,rn,rnprime,mubase,szigma) {    

# Calculates the probability of rejecting a marker on two stage
# pooled data.
#
#  The probability table corresponding to the marker is characterized by p1 and po3.
#  c and cs are the critical values in the first and second stage, respectively.
#  rn and rnprime are the sample sizes in the first and second stage, respectively.
#
# PACKAGE NEEDED: mvtnorm
# FUNCTION NEEDED: -


      c <- sqrt(csquare)
      ccsill <- sqrt(ccsillsquare)

      r <- sqrt(rn/(rn+rnprime))

      mu <- mubase * sqrt(rn)
      mucsill <- mubase * sqrt(rn+rnprime)

      x1 <- (c-mu)/szigma
      x2 <- (-c-mu)/szigma
      y1 <- (ccsill-mucsill)/szigma
      y2 <- (-ccsill-mucsill)/szigma


# The calculation

      sigma<-diag(2)

      sigma[2,1]<-r
      sigma[1,2]<-r

      prob<-rep(0,4)
      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(x1,y1), upper=c(Inf,Inf))
      prob[1] <- resa[1]

      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(-Inf,y1), upper=c(x2,Inf))
      prob[2] <- resa[1]

      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(x1,-Inf), upper=c(Inf,y2))
      prob[3] <- resa[1]

      resa <- pmvnorm(mean=rep(0,2), sigma, lower=c(-Inf,-Inf), upper=c(x2,y2))
      prob[4] <- resa[1]

      value <- sum(prob)

      value

}