optim2by3nonpooled <- function(p,p1,p2,q1,q2,p0,ptd,fdr) {

#
# This function computes the first and second stage critical values and sample sizes of the two-stage 
# design for genotype-based test (2X3 tables) with minimal genotyping burden for given PTD 
# and pFDR, when data are NOT POOLED. The calculations are based on the COST-EFFECTIVE APPROXIMATION.
#
#
# This function uses an algorithm given in the article
#  "J\'{o}zsef Buksz\'{a}r and Edwin J. C. G. van den Oord
#   Optimization of two-stage genetic designs where data are pooled 
#   usingan accurate and efficient approximation for Pearson's statistic,
#   Biometrics, 2005 (submitted)"
#
# Questions about the codes should be addressed to
#  Jo'zsef Buksza'r 
#  email: jbukszar@vcu.edu
#
# 
#
#       p  =  proportion of controls in the sample size
#   p1, p2 =  probability that a control falls in category 1, 2, respectively, under the alternative hypothesis
#   q1, q2 =  probability that a case falls in category 1, 2, respectively, under the alternative hypothesis
#       p0 =  proportion of true null hypotheses
#      ptd =  proportion of true discoveries (PTD), which can be interpreted as average power
#      fdr =  positive false discovery rate (pFDR).
#
#  PACKAGE NEEDED: none
#  FUNCTION NEEDED:  myoptimalnonnewCA, critcalNP, refineNP (directly)
#              n2minsimpNP, Gcritsimp, sumchisq (indirectly) 
#              condevalnonsimpCA, rminimizernonCA, GcritsimpCA (indirectly)
#                                     
#                  


  q <- 1 - p
  p3 <- 1-p1-p2
  q3 <- 1-q1-q2


  # Calculating mubase1, mubase2, szig1 and szig2
  #   mubase1 is the SQUARE of the expected value of normal random variable Z_1 divided by n 
  #   szig1 is the standard deviation of normal random variable Z_1,
  #   where Z_1*Z_1 + Z_2*Z_2 is the cost-effective approximation.

  P <- c(p1, p2, p3)
  Q <- c(q1, q2, q3)

  B <- rep(0,9)

  qux <- q*(P-Q)/ ( 2*(p*P+q*Q) )
  qux = 1 + qux

  pux <- p*(P-Q)/ ( 2*(p*P+q*Q) )
  pux = 1 - pux


  icount <- 1
  for (i in 1:3) {
      for (j in 1:3) {
          B[icount] <- - (qux[i]*qux[j]*p*Q[i]*Q[j] + pux[i]*pux[j]*q*P[i]*P[j]) / sqrt( (P[i]*p+Q[i]*q)*(P[j]*p+Q[j]*q) )
          icount = icount + 1
      }
  }


  icount <- 1
  for (i in 1:3) {
      B[icount] = ( qux[i]*qux[i]*p*Q[i]*(1-Q[i]) + pux[i]*pux[i]*q*P[i]*(1-P[i]) ) /  (P[i]*p+Q[i]*q)
      icount = icount + 4
  }


  B <- cbind( B[1:3], B[4:6], B[7:9])

  e <- eigen( B, symmetric=TRUE )
  eig <- e$values
  eigvect <- e$vectors

  exporig <- rep(0,3)
  for (i in 1:3) {
      exporig[i] = (Q[i] - P[i]) / sqrt(P[i]*p+Q[i]*q) 
  }
  exporig = sqrt(p*q)*exporig

  expeig <- rep(0,3)
  for (i in 1:3) {
      expeig[i] = sum(exporig*eigvect[,i]) 
  }

  ordeig <- order(eig)


  szig1 <- sqrt( eig[ordeig[3]] )
  szig2 <- sqrt( eig[ordeig[2]] )
  mubase1 <- expeig[ordeig[3]]*expeig[ordeig[3]] +  (expeig[ordeig[1]]*expeig[ordeig[1]]*0.5)
  mubase2 <- expeig[ordeig[2]]*expeig[ordeig[2]] +  (expeig[ordeig[1]]*expeig[ordeig[1]]*0.5)


  # End of mubase and szigma calculations


   res <- myoptimalnonnewCA(p,p1,p2,q1,q2,p0,ptd,fdr)

   c100 <- res[2]
   c200 <- res[3]
    n10 <- round(res[4])
    n20 <- round(res[5])

   res <- critcalNP(c100,c200,n10,n20,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)

   c10 <- res[1]
   c20 <- res[2]
   gb0 <- res[3]

   print( " Initial genburd, c1, c2, n1, n2 = " )
   print( c(gb0, c10, c20, n10, n20) )

print("Refinment starts")

    refineNP(c10,c20,n10,n20,p,p1,p2,q1,q2,p0,ptd,fdr)

}



myoptimalnonnewCA <- function(p,p1,p2,q1,q2,p0,ptd,fdr) {    

# FOR 2 X 3 TABLES!
# It finds the minimal genotyping burden, calculated by "condevalnonsimpCA",
# based on the unconstrained NLP. 
# It optimizes on the variables c (first stage critical value) and r (=sqrt(rn/(rn+rnprime)).
#
# PACKAGE NEEDED: -
# FUNCTION NEEDED: condevalnonsimpCA, rminimizernonCA, (directly)
#                  GcritsimpCA (indirectly)

  q <- 1 - p
  p3 <- 1 - p1 - p2
  q3 <- 1 - q1 - q2


  # Calculating noncbase
  #   noncbase is the non-centrality parameter of the approximating chisquared
  #   random variable with d.f. 2, divided by n

  P <- c(p1, p2, p3)
  Q <- c(q1, q2, q3)

  noncbase <- 0
  for (i in 1:3) {
      aux <- P[i]-Q[i]
      noncbase = noncbase + aux*aux/(p*P[i]+q*Q[i])
  }

  noncbase = p*q*noncbase

  # End of noncbase calculations


  aux <- (ptd*fdr) / (p0*(1-fdr))
  b <- aux * (1-p0)
  aux = qnorm(0.5*b)
  cmax <- aux * aux

# --------------------------

  cold <- 4.7
  r0 <- 0.35
  cdelta <- 0.00001

  old <- 0
  new <- 0
  cnew <- 0

  cveryold <- 0
  veryold <- 0


  res <- rminimizernonCA(cold,r0,p0,ptd,fdr,noncbase)   
  old = res[1]
  r0 = res[2]
  res <- rminimizernonCA(cold+cdelta,r0,p0,ptd,fdr,noncbase) 
  new = res[1]

  if (new < old) cstep <- 0.5 else cstep <- -0.5
  cnew = cold + cstep
  res <- rminimizernonCA(cnew,r0,p0,ptd,fdr,noncbase) 
  new = res[1]

  cveryold = cold
  veryold = old
  while (new < old) {
      cveryold = cold
      veryold = old
      cold = cnew
       old = new
      cnew = cold + cstep
      res <- rminimizernonCA(cnew,r0,p0,ptd,fdr,noncbase) 
      new = res[1]
      r0 = res[2]
  }

  if (cveryold < cnew) {
     a <- cveryold
     b <- cnew
     fva <- veryold
     fvb <- new
  } else {
     b <- cveryold
     a <- cnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  res <- rminimizernonCA(c,r0,p0,ptd,fdr,noncbase) 
  fvc <- res[1]

  iter <- 0
  while ( b-a > 0.00001 ) {
     iter = iter + 1

     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        res <- rminimizernonCA(c,r0,p0,ptd,fdr,noncbase)
        fvc = res[1]
        r0 = res[2]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            res <- rminimizernonCA(c,r0,p0,ptd,fdr,noncbase)
            fvc = res[1]
            r0 = res[2]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            res <- rminimizernonCA(a,r0,p0,ptd,fdr,noncbase)
            fva = res[1]
            res <- rminimizernonCA(b,r0,p0,ptd,fdr,noncbase)
            fvb = res[1]
            r0 = res[2]
         }
     }    

  } # end while

  if ( fvc > fva ) {
     crit1 <- a
     gb <- fva
  } else {
      if ( fvc > fvb ) {
          crit1 <- b
          gb <- fvb
      } else {
          crit1 <- c
          gb <- fvc
      }
  }    

  resul <- rminimizernonCA(crit1,r0,p0,ptd,fdr,noncbase)
  rold <- resul[2]
  res <- condevalnonsimpCA(crit1,rold,p0,ptd,fdr,noncbase)

  res

}



critcalNP <- function(c10,c20,n1,n2,mubase1,mubase2,szig1,szig2,p0,ptd,fdr) {

#  FOR 2 X 3 TABLES
#  BASED ON COMMON APPROACH
#  Calculates the first and second stage critical values for the 
#  first and second stage sample sizes so that both constraints hold. 
#
#  PACKAGE NEEDED: -
#  FUNCTION NEEDED: Gcritsimp (directly)
#                   sumchisq (indirectly)
#                  

    r <- sqrt(n1/(n1+n2))

    a <- ptd
    aux <- (ptd*fdr) / (p0*(1-fdr))
    b <- aux * (1-p0)


    preject1 <- function(c1x) { 
        c2x <- qchisq(1 - ( b / (1-pchisq(c1x,2)) ),2)
        value <- Gcritsimp(c1x,n1,mubase1,mubase2,szig1,szig2) * Gcritsimp(c2x,n2,mubase1,mubase2,szig1,szig2)  - a
        value
    }


    lo <- preject1(1)
    up <- preject1(13)
    if (lo*up>0 || is.nan(lo*up)) {
        print(" PROBLEM FOUND IN critcalNP: UNIROOT CANNOT BE USED!!!!!")
        print(c(lo,up))
    } else {
      o   <- uniroot(preject1, lower = 1, upper = 13, tol = 0.00001)
      c1 <- o$root[1]  
      c2 <- qchisq(1 - ( b / (1-pchisq(c1,2)) ),2)  

      genburd <- n1 + n2*( p0*(1-pchisq(c1,2)) +(1-p0)*Gcritsimp(c1,n1,mubase1,mubase2,szig1,szig2) ) 
    }        


    value <- c(c1,c2,genburd)

    value
    

}



refineNP <- function(c10,c20,n10,n20,p,p1,p2,q1,q2,p0,ptd,fdr) {


#  FOR 2 X 3 TABLES
#  BASED ON COST-EFFECTIVE METHOD
#
#  PACKAGE NEEDED: -
#  FUNCTION NEEDED: n2minsimpNP, critcalNP
#                   Gcritsimp, sumchisq (indirectly)
#                  


  q <- 1 - p
  p3 <- 1-p1-p2
  q3 <- 1-q1-q2


  # Calculating mubase1, mubase2, szig1 and szig2
  #   mubase1 is the SQUARE of the expected value of normal random variable Z_1 divided by n 
  #   szig1 is the standard deviation of normal random variable Z_1,
  #   where Z_1*Z_1 + Z_2*Z_2 is the cost-effective approximation.

  P <- c(p1, p2, p3)
  Q <- c(q1, q2, q3)

  B <- rep(0,9)

  qux <- q*(P-Q)/ ( 2*(p*P+q*Q) )
  qux = 1 + qux

  pux <- p*(P-Q)/ ( 2*(p*P+q*Q) )
  pux = 1 - pux


  icount <- 1
  for (i in 1:3) {
      for (j in 1:3) {
          B[icount] <- - (qux[i]*qux[j]*p*Q[i]*Q[j] + pux[i]*pux[j]*q*P[i]*P[j]) / sqrt( (P[i]*p+Q[i]*q)*(P[j]*p+Q[j]*q) )
          icount = icount + 1
      }
  }


  icount <- 1
  for (i in 1:3) {
      B[icount] = ( qux[i]*qux[i]*p*Q[i]*(1-Q[i]) + pux[i]*pux[i]*q*P[i]*(1-P[i]) ) /  (P[i]*p+Q[i]*q)
      icount = icount + 4
  }


  B <- cbind( B[1:3], B[4:6], B[7:9])

  e <- eigen( B, symmetric=TRUE )
  eig <- e$values
  eigvect <- e$vectors

  exporig <- rep(0,3)
  for (i in 1:3) {
      exporig[i] = (Q[i] - P[i]) / sqrt(P[i]*p+Q[i]*q) 
  }
  exporig = sqrt(p*q)*exporig

  expeig <- rep(0,3)
  for (i in 1:3) {
      expeig[i] = sum(exporig*eigvect[,i]) 
  }

  ordeig <- order(eig)


  szig1 <- sqrt( eig[ordeig[3]] )
  szig2 <- sqrt( eig[ordeig[2]] )
  mubase1 <- expeig[ordeig[3]]*expeig[ordeig[3]] +  (expeig[ordeig[1]]*expeig[ordeig[1]]*0.5)
  mubase2 <- expeig[ordeig[2]]*expeig[ordeig[2]] +  (expeig[ordeig[1]]*expeig[ordeig[1]]*0.5)

# End of mubase and szigma calculations


  old <- 0
  new <- 0
  rnew <- 0

  rold <- n10
  rveryold <- 0
  veryold <- 0


  res <- n2minsimpNP(c10,c20,rold,n20,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  old = res[1]

  res <- n2minsimpNP(c10,c20,rold+1,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  new = res[1]

  if (new < old) rstep <- 4 else rstep <- -4
  rnew = rold + rstep
  res <- n2minsimpNP(c10,c20,rnew,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  new = res[1]


  rveryold = rold
  veryold = old
#print(rstep)
print( "################" )
print( c("####",rveryold,rold,rnew) )
print( c("####",veryold,old,new) )

  while (new < old) {
      rveryold = rold
      veryold = old
      rold = rnew
       old = new
      rnew = rold + rstep
      res <- n2minsimpNP(c10,c20,rnew,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
      new = res[1]

print( "################" )
print( c("####",rveryold,rold,rnew) )
print( c("####",veryold,old,new) )
  }

  if (rveryold < rnew) {
     a <- rveryold
     b <- rnew
     fva <- veryold
     fvb <- new
  } else {
     b <- rveryold
     a <- rnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  res <- n2minsimpNP(c10,c20,c,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  fvc <- res[1]

  iter <- 0
  while ( b-a > 2 ) {
     iter = iter + 1
     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        res <- n2minsimpNP(c10,c20,c,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
        fvc = res[1]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            res <- n2minsimpNP(c10,c20,c,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
            fvc = res[1]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            res <- n2minsimpNP(c10,c20,a,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
            fva = res[1]
            res <- n2minsimpNP(c10,c20,b,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
            fvb = res[1]
         }
     } 

print( c("#### ", iter, " ####") )
print( c("####",a,c,b) )
print( c("####",fva,fvc,fvb) )

  } # end while

  if ( fvc > fva ) {
     value <- c(fva,a)
  } else {
      if ( fvc > fvb ) {
          value <- c(fvb,b)
      } else {
          value <- c(fvc,c)
      }
  }    

  genburd <- value[1]
  n1 <- value[2]
  res <- n2minsimpNP(c10,c20,n1,res[2],mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  n2 = res[2]
  res <- critcalNP(c10,c20,n1,n2,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  c1 <- res[1]
  c2 <- res[2]

  print(" END RESULT: ")

  print(" First and second stage critical value = ")
  print( c(c1, c2) )

  print(" First and second stage sample size = ")
  print( c(n1, n2) )

  print(" genotyping burden = ")
  print( genburd )

}


n2minsimpNP <- function(c10,c20,n1,n20,mubase1,mubase2,szig1,szig2,p0,ptd,fdr) {

#  FOR 2 X 3 TABLES
#  BASED ON COST-EFFECTIVE METHOD
#
#  PACKAGE NEEDED: -
#  FUNCTION NEEDED: critcalNP (directly)
#                   Gcritsimp, sumchisq (indirectly)                  


  old <- 0
  new <- 0
  rnew <- 0

  rold <- n20
  rveryold <- 0
  veryold <- 0

  res <- critcalNP(c10,c20,n1,rold,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  old = res[3]

  res <- critcalNP(c10,c20,n1,rold+1,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  new = res[3]

  if (new < old) rstep <- 8 else rstep <- -8
  rnew = rold + rstep
  res <- critcalNP(c10,c20,n1,rnew,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  new = res[3]

#print(rstep)

  rveryold = rold
  veryold = old
  while (new < old) {
      rveryold = rold
      veryold = old
      rold = rnew
       old = new
      rnew = rold + rstep
      res <- critcalNP(c10,c20,n1,rnew,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
      new = res[3]

#print( "***********" )
#print( c(rveryold,rold,rnew) )
#print( c(veryold,old,new) )
  }

  if (rveryold < rnew) {
     a <- rveryold
     b <- rnew
     fva <- veryold
     fvb <- new
  } else {
     b <- rveryold
     a <- rnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  resc <- critcalNP(c10,c20,n1,c,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
  fvc <- resc[3]

  iter <- 0
  while ( b-a > 2 ) {
     iter = iter + 1
     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        resc <- critcalNP(c10,c20,n1,c,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
        fvc = resc[3]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            resc <- critcalNP(c10,c20,n1,c,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
            fvc = resc[3]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            resa <- critcalNP(c10,c20,n1,a,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
            fva = resa[3]
            resb <- critcalNP(c10,c20,n1,b,mubase1,mubase2,szig1,szig2,p0,ptd,fdr)
            fvb = resb[3]
         }
     } 

#print( c("-----  ", iter, "  ------") )
#print( c(a,c,b) )
#print( c(fva,fvc,fvb) )

  } # end while

  if ( fvc > fva ) {
     value <- c(fva,a)
  } else {
      if ( fvc > fvb ) {
          value <- c(fvb,b)
      } else {
          value <- c(fvc,c)
      }
  }    

 
  value

}



Gcritsimp <- function(c,n,mubase1,mubase2,szig1,szig2) {    

# FOR 2 X 3 TABLES!
# BASED ON THE COST-EFFECTIVE APPROXIMATION!
# Calculates the probability of rejecting a marker under the alternative hypothesis in a single 
# stage when the critical value is c, the sample size is n.
# Parameters specifying the probability table p1, p2, q1 and q2 as well as th proportion of cases p
# are wrapped in mubase1, mubase2, szig1 and szig2. 
#
# FUNCTION NEEDED: sumchisq
     

    lam1 <- szig1 * szig1
    lam2 <- szig2 * szig2

    powac <- sumchisq( c,c(lam1,lam2),c(mubase1*n/lam1,mubase2*n/lam2),5,0.2,1e-08 )

    power <- 1 - powac[1]

    power

}



sumchisq <- function(y,alfa,del,beta,mu0,accur) {


#  Calculates the distribution function value at y,
#  where the distribution is the linear compination
#  of d.f.1 noncentral chisquares.
#
#  This function uses an algorithm given in the paper
#  "Casta\~{n}o-Mart\'{i}nez, A. and L\'{o}pez-Bl\'{a}zquez, F.,
#   Distribution of a sum of weighted noncentral chi-square variables,
#   Test, 2005 (in press)."
#
#  INPUT PARAMETERS:
#    del = array containing the noncentrality parameters of the chisquared random variables.
#   alfa = array containing the coefficients of the chisquared random variables in the sum.
#   beta = constant not equal to any of alfa's
#  accur = the required accuracy
#
#  FUNCTIONS NEEDED: -

   n <- length(alfa) # the number of chisquared random variables.

   N <- 1000

   c <- rep(0,(N+1)) # c[k+1] = c_k !!!
   d <- rep(0,N)

   p <- n/2 + 1

   aux <- -mu0 / (p-mu0)
   r <- 1 / ( beta*mu0+(p-mu0)*alfa )
   e <- mu0*r
   delal <- del*alfa
   t <- (p-mu0)*delal*r
   f <- beta-alfa
   ef <- e*f
   coeff <- 0.5*beta*p/mu0


   d1 <- -coeff*delal*e*e
   d2 <- aux
   d3 <- 0.5*ef
   d[1] = sum(d1) + d2 + sum(d3)

   yh <- 0.5*y/beta

   L <- rep(0,N) #  Laguerre polynomial, array L[j+1] = L_j(x) with j=0,1,...
   x <- p*yh/mu0
   alf <- p-1
   L[1] = 1 
   L[2] = 2 + alf - 1 - x 

   c[1] = 2*((beta*p)**p)*sqrt(prod(r))*exp(-0.5*sum(t)) / (p-mu0)

   sum <- 0
   paux <- 1
   k <- 0
   coe <- exp(-yh)*(yh**(p-1)) / ( 2*beta*factorial(p-1) )
   oldc <- 0
   newc <- 0.5*c[1]*L[1]
   while ( abs(oldc) < abs(newc) ) {
       oldc = newc
       newc = c[k+1]*factorial(k)*L[k+1]/paux
       sum = sum + newc
       paux = paux*(p+k)

       k = k + 1
       sumi <- 0
       for (j in 0:(k-1) ) sumi = sumi + c[j+1]*d[k-j]
       c[k+1] = sumi/k   

       d1 = ef*d1
       d2 = aux*d2
       d3 = ef*d3
       d[k+1] = (k+1)*sum(d1) + d2 + sum(d3)

       L[k+2] = ( (2*(k+1)+alf-1-x)*L[k+1] - (k+alf)*L[k] ) / (k+1)

   }

   while (abs(coe*newc) > accur && k <= N) {
       newc = c[k+1]*factorial(k)*L[k+1]/paux
       sum = sum + newc
       paux = paux*(p+k)

       k = k + 1
       sumi <- 0
       for (j in 0:(k-1) ) sumi = sumi + c[j+1]*d[k-j]
       c[k+1] = sumi/k   

       d1 = ef*d1
       d2 = aux*d2
       d3 = ef*d3
       d[k+1] = (k+1)*sum(d1) + d2 + sum(d3)

       L[k+2] = ( (2*(k+1)+alf-1-x)*L[k+1] - (k+alf)*L[k] ) / (k+1)

   }

   res <- c(coe * sum,coe * newc)

   res
   
}


condevalnonsimpCA <- function(c,r,p0,ptd,fdr,noncbase) {    

# FOR 2 X 3 TABLES
# 
# ASSUMES NONPOOLED DATA
# For a first stage critical value and r=sqrt(rn/rntot), it calculates 
# the second stage critical value, the first and second stage sample size 
# and the genotyping burden in such a way that the two constraints in the NLP hold. 
#
# PACKAGE NEEDED: -
# FUNCTION NEEDED:  GcritsimpCA

    a <- ptd
    aux <- (ptd*fdr) / (p0*(1-fdr))
    b <- aux * (1-p0)

    rcoeff <- 1/(r*r) - 1

# CALCULATING THE SECOND STAGE CRITICAL VALUE: BASED ON THE SECOND CONSTRAINT

    aux <-  b / (1-pchisq(c,2))
    cs <- qchisq(1 - aux,2)

# CALCULATING THE FIRST AND THE SECOND STAGE SAMPLE SIZE: BASED ON THE FIRST CONSTRAINT


    preject1 <- function(nx) { 
        npx <- rcoeff*nx
        value <- ( 1 - pchisq(c,2,noncbase*nx) ) * ( 1 - pchisq(cs,2,noncbase*npx) )  - a
        value
    }
    lo <- preject1(1)
    up <- preject1(100000)
    if (lo*up>0 || is.nan(lo*up)) {
        print(" PROBLEM FOUND: UNIROOT CANNOT BE USED!!!!!")
        print(c(lo,up))
        print(c(c,r,cs))
        genburd <- 10000000
    } else {
      o   <- uniroot(preject1, lower = 1, upper = 100000, tol = 0.00001)
      rn <- o$root[1]  # The second condition holds with this rn (first stage sample size) and 
      rnprime <- rcoeff * rn           # this rnprime (second stage sample size).

      genburd <- rn + rnprime*( p0*(1-pchisq(c,2)) +(1-p0)*GcritsimpCA(c,rn,noncbase) ) 

    }
                                           
    value <- c(genburd,c,cs,rn,rnprime)

    value

}



rminimizernonCA <- function(cold,r0,p0,ptd,fdr,noncbase) { 

#  FOR 2 X 3 TABLES   
# For a given first stage critical value, cold, it calculates r
# that minimizes the genotyping burden in the unconstrained NLP.
#
# PACKAGE NEEDED: -
# FUNCTION NEEDED: condevalnonsimpCA
#                  GcritsimpCA (indirectly)


  rdelta <- 0.000001

  old <- 0
  new <- 0
  rnew <- 0

  rold <- r0
  rveryold <- 0
  veryold <- 0

  res <- condevalnonsimpCA(cold,rold,p0,ptd,fdr,noncbase)
  old = res[1]
  res <- condevalnonsimpCA(cold,rold+rdelta,p0,ptd,fdr,noncbase)
  new = res[1]

  if (new < old) rstep <- 0.1 else rstep <- -0.1
  rnew = rold + rstep
  res <- condevalnonsimpCA(cold,rnew,p0,ptd,fdr,noncbase)
  new = res[1]

  rveryold = rold
  veryold = old
  while (new < old) {
      rveryold = rold
      veryold = old
      rold = rnew
       old = new
      rnew = rold + rstep
      res <- condevalnonsimpCA(cold,rnew,p0,ptd,fdr,noncbase)
      new = res[1]
  }

  if (rveryold < rnew) {
     a <- rveryold
     b <- rnew
     fva <- veryold
     fvb <- new
  } else {
     b <- rveryold
     a <- rnew
     fvb <- veryold
     fva <- new
  }

  c <- (a+b)*0.5
  res <- condevalnonsimpCA(cold,c,p0,ptd,fdr,noncbase)
  fvc <- res[1]

  iter <- 0
  while ( b-a > 0.00001 ) {
     iter = iter + 1
     if ( fvc > fva ) {
        b = c
        fvb = fvc
        c = (a+b)*0.5
        res <- condevalnonsimpCA(cold,c,p0,ptd,fdr,noncbase)
        fvc = res[1]
     } else {
         if ( fvc > fvb ) {
            a = c
            fva = fvc
            c = (a+b)*0.5
            res <- condevalnonsimpCA(cold,c,p0,ptd,fdr,noncbase)
            fvc = res[1]
         } else {
            a = (a+c)*0.5
            b = (b+c)*0.5
            res <- condevalnonsimpCA(cold,a,p0,ptd,fdr,noncbase)
            fva = res[1]
            res <- condevalnonsimpCA(cold,b,p0,ptd,fdr,noncbase)
            fvb = res[1]
         }
     }    

  } # end while

  if ( fvc > fva ) {
     value <- c(fva,a)
  } else {
      if ( fvc > fvb ) {
          value <- c(fvb,b)
      } else {
          value <- c(fvc,c)
      }
  }    

  value

}



GcritsimpCA <- function(c,n,noncbase) {    

# FOR 2 X 3 TABLES!
#
# Calculates the probability of rejecting a marker under the alternative hypothesis in a single 
# stage when the critical value is c, the sample size is n.
# Parameters specifying the probability table p1, p2, q1 and q2 as well as the proportion of cases p
# are wrapped in noncbase. 
#
# FUNCTION NEEDED: -
     
    power <- 1 - pchisq(c,df=2,ncp=n*noncbase)

    power

}






